#include <GLUT/glut.h>									//GLUT library
#include "UniformGrid.h"
#include "GlyphRenderer.h"
#include "DirCodingRenderer.h"
#include <iostream>										
#include <math.h>										//For expf()



float Xmin	= -5, Xmax	= 5;							//Range of variable x that we sample
float Ymin	= -5, Ymax	= 5;							//Range of variable y that we sample
int   Nx	= 40, Ny	= 40;							//Number of samples taken along the x and y axes



float f(float x, float y)								//A simple two-variable function to plot 
{														//The function is samples 
	return 8*expf(-(x*x+y*y)/5);
}


UniformGrid			grid(Nx,Ny,Xmin,Ymin,Xmax,Ymax);
GlyphRenderer*		glyph_renderer = 0;
DirCodingRenderer*	dir_renderer   = 0;
int					scaling_mode   = 0;
float				alpha          = 0;
int					draw_glyphs    = 1;



void viewing(int W, int H)								//Window resize function, sets up viewing parameters (GLUT callback function)
{
	glMatrixMode(GL_MODELVIEW);							//1. Set the modelview matrix (including the camera position and view direction)
	glLoadIdentity();										

	glMatrixMode(GL_PROJECTION);						//2. Set the projection matrix
	glLoadIdentity();
	gluOrtho2D(Xmin,Xmax,Ymin,Ymax);
	glDisable(GL_DEPTH_TEST);

	glViewport(0,0,W,H);								//3. Set the viewport to the entire size of the rendering window
}



void draw()												//Render the grid (GLUT callback function)
{
    if (draw_glyphs)									//Depending on 'draw_glyphs', show the vectors either with glyphs
		glyph_renderer->draw(grid);						//or with directional color-coding
	else
		dir_renderer->draw(grid);
}	


void createGrid(float alpha)							//Create vectors for the grid. For this, we simply sample the 
{														//rotated gradient of the function f(x,y) at all the grid points
	for(int i=0;i<grid.numPoints();++i)					
	{
	   float p[2];				
	   grid.getPoint(i,p);								//Get coordinates of current grid point
	   
	   float scalar = f(p[0],p[1]);						//Evaluate f(x,y) at current point
		
	   float delta    = 0.001*(Xmax-Xmin);				//Evaluate gradient of f(x,y) using finite differences
	   float scalar_x = f(p[0]+delta,p[1]);	
	   float scalar_y = f(p[0],p[1]+delta);		   
	   Point3d gradient = Point3d((scalar_x-scalar)/delta,(scalar_y-scalar)/delta,0);

	   float alpha2 = sin(alpha)*gradient.norm();		//Use a fancy rotation function!
	   
	   Point3d vec;										//Rotate the gradient with an angle alpha2, to generate more complex vector fields
	   vec.x = cos(alpha2)*gradient.x - sin(alpha2)*gradient.y;
	   vec.y = sin(alpha2)*gradient.x + cos(alpha2)*gradient.y;
	   	   
	   grid.pointVectors().setC0Vector(i,vec);			//Store the rotated gradient as vector values in the grid
	}   
}



void keyboard(unsigned char c,int,int)					//Keyboard callback
{
	switch(c)
	{
	case 'v':											//Change the vector field to visualize
		{
		  alpha = (alpha + M_PI/50);
		  createGrid(alpha);
		  break;
		}
	case ' ':											//Toggle between glyphs and directional color-coding
		{
		  draw_glyphs = !draw_glyphs;
		  break;
		}  		
	}
				
	glutPostRedisplay();    
}	



int main(int argc,char* argv[])								//Main program
{
   cout<<"6. Directional color coding (2D vector fields)"<<endl;
   cout<<"In the graphics window, press"<<endl;
   cout<<"  space: show glyphs or directional color-coding "<<endl;
   cout<<"  v:     change the vector field"<<endl<<endl;
   cout<<"Directional color-coding legend:"<<endl;
   cout<<"  red:     pointing to the right (east)"<<endl;
   cout<<"  yellow:  pointing north-east"<<endl;
   cout<<"  green:   pointing north-west"<<endl;
   cout<<"  cyan:    pointing to the left (west)"<<endl;
   cout<<"  blue:    pointing south-west"<<endl;
   cout<<"  magenta: pointing south-east"<<endl;

   glutInit(&argc, argv);									//1.  Initialize the GLUT toolkit
   glutInitDisplayMode(GLUT_RGB| GLUT_DOUBLE|GLUT_DEPTH);	//2.  Ask GLUT to create next windows with a RGB framebuffer and a Z-buffer too
   glutInitWindowSize(500,500);								//3.  Tell GLUT how large are the windows we want to create next
   glutCreateWindow("5. Directional color coding (2D vector fields)");		
															//4.  Create our window
   
   
   glyph_renderer = new GlyphRenderer();					//5.  Create a glyph renderer, and initialize its various parameters
   float cell_size = (grid.getCellSize1()+grid.getCellSize2())/2.0;	
   glyph_renderer->setDrawingStyle(GlyphRenderer::DRAW_CONES);
   glyph_renderer->setScalingFactor(cell_size);
   glyph_renderer->setColormapType(ColormapRenderer::COLORMAP_SATURATION);
   glyph_renderer->setScalingMode(GlyphRenderer::SCALE_NORMALIZE);


   dir_renderer   = new DirCodingRenderer();				//6.  Create a direction-coloding renderer
   
   createGrid(alpha);										//7.  Create vector values for the grid vertices

   
   glutKeyboardFunc(keyboard);   
   glutDisplayFunc(draw);									//8.  Add a drawing callback to the window	
   glutReshapeFunc(viewing);								//9.  Add a resize callback to the window
   glutMainLoop();											//10. Start the event loop that displays the graph and handles window-resize events
   
   return 0;
}




